@@ -32,7 +32,6 @@
 #' @param ncpus integer; number of CPUs for parallel calculations (default: 1).
 #' @param plots logical; whether to produce histogram and density plots.
 #' @export
-#' @importFrom boot boot boot.ci
 #' @importFrom grDevices rgb
 #' @importFrom graphics hist layout
 #' @importFrom evd dgpd pgpd qgpd
@@ -44,8 +43,16 @@
 #' fit.extgp(rain[rain>0], model=1, method = 'mle', init = c(0.9, gp.fit(rain, 0)$est),
 #'  rounded = 0.1, confint = TRUE, R = 20)
 #' }
-fit.extgp <- function(data, model = 1, method = c("mle", "pwm"), init, censoring = c(0, Inf),
-                      rounded = 0, confint = FALSE, R = 1000, ncpus = 1, plots = TRUE) {
+fit.extgp <- function(data,
+                      model = 1,
+                      method = c("mle", "pwm"),
+                      init,
+                      censoring = c(0, Inf),
+                      rounded = 0,
+                      confint = FALSE,
+                      R = 1000,
+                      ncpus = 1,
+                      plots = TRUE) {
     method <- match.arg(method, c("mle", "pwm"), several.ok = TRUE)
     if("pwm" %in% method){
       if (!requireNamespace("gmm", quietly = TRUE)) {
@@ -53,10 +60,16 @@ fit.extgp <- function(data, model = 1, method = c("mle", "pwm"), init, censoring
              call. = FALSE)
       }
     }
+    if(confint){
+      if (!requireNamespace("boot", quietly = TRUE)) {
+        stop("Package \"boot\" needed for this function to work. Please install it.",
+             call. = FALSE)
+      }
+    }
     # Sanity checks
     initsize <- switch(model, 3, 3, 4, 5)
     if (length(init) != initsize) {
-        stop("Invalid starting values in `init'; incorrect length.")
+        stop("Invalid starting values in \"init\"; incorrect length.")
     }
     data = data[data > 0]
     if (model == 3 && "pwm" %in% method) {
@@ -390,31 +403,58 @@ egp2.fit <- function(data, model = 1, method = c("mle", "pwm"), init, censoring
 #' @param direct logical; which method to use for sampling in model of \code{type} \code{4}?
 #' @author  Raphael Huser and Philippe Naveau
 #'
-#' @section Usage: \code{pextgp.G(u, type=1,prob, kappa, delta)}
-#' @section Usage: \code{dextgp.G(u,type=1, prob=NA, kappa=NA, delta=NA, log=FALSE)}
-#' @section Usage: \code{qextgp.G(u,type=1, prob=NA,kappa=NA,delta=NA)}
-#' @section Usage: \code{rextgp.G(n,prob=NA,kappa=NA,delta=NA,type=1,unifsamp=NULL,direct=FALSE,censoring=c(0,1))}
+#' @section Usage: \code{pextgp.G(u, type=1, prob, kappa, delta)}
+#' @section Usage: \code{dextgp.G(u, type=1, prob=NA, kappa=NA, delta=NA, log=FALSE)}
+#' @section Usage: \code{qextgp.G(u, type=1, prob=NA, kappa=NA, delta=NA)}
+#' @section Usage: \code{rextgp.G(n, prob=NA, kappa=NA, delta=NA,
+#' type=1, unifsamp=NULL, direct=FALSE, censoring=c(0,1))}
 NULL
 
 #' @rdname extgp-functions
 #' @export
 #' @keywords internal
 pextgp.G <- function(u, type = 1, prob, kappa, delta) {
+    type <- as.integer(type[1])
     if (!type %in% 1:5) {
-        stop("Invalid `type' argument")
+        stop("Invalid \"type\" argument")
     }
-    type <- type[1]
-    if (type %in% c(1, 3, 4) && missing(kappa)) {
-        stop("Argument `kappa' missing.")
+    if (type %in% c(1, 3, 4)) {
+      if(missing(kappa)){
+        stop("Argument \"kappa\" missing.")
+      } else{
+        if(!isTRUE(all(is.numeric(kappa),
+                             length(kappa) == 1L,
+                             kappa > 0))){
+          stop("Invalid \"kappa\" parameter.")
+        }
+      }
     }
-    if (type %in% c(2, 3, 4) && missing(delta)) {
-        stop("Argument `delta' missing.")
+    if (type %in% c(2, 3, 4)){
+      if(missing(delta)) {
+        stop("Argument \"delta\" missing.")
+      } else{
+        if(!isTRUE(all(is.numeric(delta),
+                       length(delta) == 1L,
+                       delta > 0))){
+          stop("Invalid \"delta\" parameter.")
+      }
+      }
     }
-    if (type == 4 && missing(prob)) {
-        stop("Argument `prob' missing.")
+    if (type == 4){
+      if(missing(prob)) {
+        stop("Argument \"prob\" missing.")
+      } else{
+        if(!isTRUE(all(is.numeric(prob),
+                       length(prob) == 1L,
+                       prob >= 0,
+                       prob <= 1))){
+          stop("Invalid \"prob\" parameter.")
+      }
+      }
     }
+    u <- pmax(pmin(u, 1), 0)
     if (type == 0) {
-        return(u)
+      return(u)
     } else if (type == 1) {
         return(u^kappa)
     } else if (type == 2) {
@@ -431,17 +471,17 @@ pextgp.G <- function(u, type = 1, prob, kappa, delta) {
 #' @keywords internal
 dextgp.G <- function(u, type = 1, prob = NA, kappa = NA, delta = NA, log = FALSE) {
     if (!type %in% 1:5) {
-        stop("Invalid `type' argument")
+        stop("Invalid \"type\" argument")
     }
     type <- type[1]
     if (type %in% c(1, 3, 4) && missing(kappa)) {
-        stop("Argument `kappa' missing.")
+        stop("Argument \"kappa\" missing.")
     }
     if (type %in% c(2, 3, 4) && missing(delta)) {
-        stop("Argument `delta' missing.")
+        stop("Argument \"delta\" missing.")
     }
     if (type == 4 && missing(prob)) {
-        stop("Argument `prob' missing.")
+        stop("Argument \"prob\" missing.")
     }
     if (log == FALSE) {
         if (type == 0) {
@@ -477,17 +517,17 @@ dextgp.G <- function(u, type = 1, prob = NA, kappa = NA, delta = NA, log = FALSE
 #' @keywords internal
 qextgp.G <- function(u, type = 1, prob = NA, kappa = NA, delta = NA) {
     if (!type %in% 1:5) {
-        stop("Invalid `type' argument")
+        stop("Invalid \"type\" argument")
     }
     type <- type[1]
     if (type %in% c(1, 3, 4) && missing(kappa)) {
-        stop("Argument `kappa' missing.")
+        stop("Argument \"kappa\" missing.")
     }
     if (type %in% c(2, 3, 4) && missing(delta)) {
-        stop("Argument `delta' missing.")
+        stop("Argument \"delta\" missing.")
     }
     if (type == 4 && missing(prob)) {
-        stop("Argument `prob' missing.")
+        stop("Argument \"prob\" missing.")
     }
     if (type == 0) {
         return(u)
@@ -513,17 +553,17 @@ qextgp.G <- function(u, type = 1, prob = NA, kappa = NA, delta = NA) {
 #' @keywords internal
 rextgp.G <- function(n, prob = NA, kappa = NA, delta = NA, type = 1, unifsamp = NULL, direct = FALSE, censoring = c(0, 1)) {
     if (!type %in% 1:5) {
-        stop("Invalid `type' argument")
+        stop("Invalid \"type\" argument")
     }
     type <- type[1]
     if (type %in% c(1, 3, 4) && missing(kappa)) {
-        stop("Argument `kappa' missing.")
+        stop("Argument \"kappa\" missing.")
     }
     if (type %in% c(2, 3, 4) && missing(delta)) {
-        stop("Argument `delta' missing.")
+        stop("Argument \"delta\" missing.")
     }
     if (type == 4 && missing(prob)) {
-        stop("Argument `prob' missing.")
+        stop("Argument \"prob missing.")
     }
     if (is.null(unifsamp)) {
         unifsamp <- runif(n)
@@ -593,6 +633,7 @@ rextgp.G <- function(n, prob = NA, kappa = NA, delta = NA, type = 1, unifsamp =
 #' @param sigma scale parameter
 #' @param xi shape parameter
 #' @param type integer between 0 to 5 giving the model choice
+#' @param step function of step size for discretization with default \code{0}, corresponding to continuous quantiles
 #' @param log logical; should the log-density be returned (default to \code{FALSE})?
 #' @param unifsamp sample of uniform; if provided, the data will be used in place of new uniform random variates
 #' @param censoring numeric vector of length 2 containing the lower and upper bound for censoring
@@ -633,8 +674,14 @@ dextgp <- function(x, prob = NA, kappa = NA, delta = NA, sigma = NA, xi = NA, ty
 #' @rdname extgp-functions
 #' @export
 #' @keywords internal
-qextgp <- function(p, prob = NA, kappa = NA, delta = NA, sigma = NA, xi = NA, type = 1) {
+qextgp <- function(p, prob = NA, kappa = NA, delta = NA, sigma = NA, xi = NA, type = 1, step = 0) {
+  stopifnot(length(step) == 1L, step >= 0, is.finite(step))
+  if(isTRUE(all.equal(step, 0, check.attributes = FALSE))){
     return(qgpd(qextgp.G(p, prob = prob, kappa = kappa, delta = delta, type = type), scale = sigma, shape = xi))
+  } else{
+    qcont <- qgpd(qextgp.G(p, prob = prob, kappa = kappa, delta = delta, type = type), scale = sigma, shape = xi)
+    return(step*floor((qcont-step)/step))
+  }
 }
 
 #' @rdname extgp-functions
